@@ -1,3 +1,4 @@
+
 /* basic info for a file or service object */
 CREATE TABLE  Services
 (
@@ -16,13 +17,19 @@ CREATE TABLE  Services
 	KeyMetadata3		Text,
 	KeyMetadata4		Text,
 	KeyMetadata5		Text,
-	KeyMetadata6		Integer,
-	KeyMetadata7		Integer,
-	KeyMetadata8		Integer,
+	KeyMetadata6		Integer, /* NOTE: If you change the type of any of these KeyMetadata%d members, */
+	KeyMetadata7		Integer, /*       then you must update the source in: */
+	KeyMetadata8		Integer, /*       tracker_data_query_metadata_field_values() */
 	KeyMetadata9		Text,
 	KeyMetadata10		Text,
 	KeyMetadata11		Text,
 
+	KeyMetadataCollation1   Text,
+	KeyMetadataCollation2   Text,
+	KeyMetadataCollation3   Text,
+	KeyMetadataCollation4   Text,
+	KeyMetadataCollation5   Text,
+
 	Icon			Text,
 	CanWrite		Integer default 1,
 	CanExecute		Integer default 1,
@@ -37,9 +44,22 @@ CREATE TABLE  Services
 	MD5			Text,
 
     	unique (Path, Name)
-
 );
 
+CREATE INDEX ServiceTypeIDIndex ON Services (ServiceTypeID);
+
+/* It would seem that sqlite is unable to use split indices for GROUP or ORDER, thus we end up
+   with this scheme where AuxilaryID is dropped from the index and ServiceType requires additional logic */
+CREATE INDEX ServicesCompoundIndex1 ON Services (ServiceTypeID, KeyMetadataCollation1, KeyMetadataCollation2);
+CREATE INDEX ServicesCompoundIndex2 ON Services (ServiceTypeID, KeyMetadataCollation2);
+CREATE INDEX ServicesCompoundIndex3 ON Services (ServiceTypeID, KeyMetadataCollation3);
+CREATE INDEX ServicesCompoundIndex4 ON Services (ServiceTypeID, KeyMetadataCollation4);
+CREATE INDEX ServicesCompoundIndex5 ON Services (ServiceTypeID, KeyMetadataCollation5);
+CREATE INDEX ServicesCompoundIndex6 ON Services (ServiceTypeID, KeyMetadata6);
+CREATE INDEX ServicesCompoundIndex7 ON Services (ServiceTypeID, KeyMetadata7);
+CREATE INDEX ServicesCompoundIndex8 ON Services (ServiceTypeID, KeyMetadata8);
+CREATE INDEX ServicesCompoundIndexAux ON Services (ServiceTypeID, AuxilaryID);
+
 /* child service relationships for a specific group/struct metadata */
 CREATE TABLE ChildServices
 (
@@ -55,13 +75,13 @@ CREATE TABLE  ServiceMetaData
 (
 	ID			Integer primary key AUTOINCREMENT not null,
 	ServiceID		Integer not null,
-	MetaDataID 		Integer  not null,
+	MetaDataID 		Integer not null,
 	MetaDataValue     	Text,
 	MetaDataDisplay		Text,
 	MetaDataCollation	Text
 );
 
-CREATE INDEX ServiceMetaDataCompoundIndex ON ServiceMetaData (ServiceID, MetaDataID, MetaDataDisplay, MetaDataCollation);
+CREATE INDEX ServiceMetaDataCompoundIndex ON ServiceMetaData (ServiceID, MetaDataID);
 
 /* metadata for all keyword types - keywords are db indexed for fast searching - they are also not processed like other metadata. */
 CREATE TABLE  ServiceKeywordMetaData 
@@ -72,7 +92,7 @@ CREATE TABLE  ServiceKeywordMetaData
 	MetaDataValue		Text COLLATE NOCASE
 );
 
-CREATE INDEX ServiceKeywordMetaDataCompoundIndex ON ServiceKeywordMetaData (ServiceID, MetaDataID, MetaDataValue);
+CREATE INDEX ServiceKeywordMetaDataCompoundIndex ON ServiceKeywordMetaData (ServiceID, MetaDataID);
 
 /* metadata for all integer/date types */
 CREATE TABLE  ServiceNumericMetaData 
