@@ -61,6 +61,9 @@ static gpointer    field_type_enum_class;
 /* Category - subcategory ids cache */
 static GHashTable *subcategories_cache;
 
+/* List of parent services in the hierarchy */ 
+static GSList     *parent_services;
+
 static void
 ontology_mime_prefix_foreach (gpointer data,
 			      gpointer user_data)
@@ -199,6 +202,14 @@ tracker_ontology_shutdown (void)
 		service_mime_prefixes = NULL;
 	}
 
+	if (parent_services) {
+		g_slist_foreach (parent_services,
+				 (GFunc) g_object_unref,
+				 NULL);
+		g_slist_free (parent_services);
+		parent_services = NULL;
+	}
+
 	g_type_class_unref (field_type_enum_class);
 	field_type_enum_class = NULL;
 
@@ -222,13 +233,17 @@ tracker_ontology_service_add (TrackerService *service,
 	name = tracker_service_get_name (service);
 
 	g_hash_table_insert (service_names,
-			     /* g_utf8_collate_key (name, -1),	*/
 			     g_strdup (name),
 			     g_object_ref (service));
 	g_hash_table_insert (service_ids,
 			     g_strdup_printf ("%d", id),
 			     g_object_ref (service));
 
+	if (tracker_service_get_parent (service) == NULL ||
+	    g_strcmp0 (tracker_service_get_parent (service), " ") == 0) {
+		parent_services = g_slist_prepend (parent_services, g_object_ref (service));
+	}
+
 	for (l = mimes; l && l->data; l = l->next) {
 		g_hash_table_insert (mimes_to_service_ids,
 				     l->data,
@@ -241,7 +256,7 @@ tracker_ontology_service_add (TrackerService *service,
 		service_mime_prefix->service = id;
 
 		service_mime_prefixes = g_slist_prepend (service_mime_prefixes,
-						       service_mime_prefix);
+							 service_mime_prefix);
 	}
 }
 
@@ -516,6 +531,12 @@ tracker_ontology_get_subcategory_ids (const gchar *service_str)
 	return subcategories;
 }
 
+GSList *        
+tracker_ontology_get_parent_services (void)
+{
+	return parent_services;
+}
+
 /*
  * Service data
  */
@@ -623,6 +644,20 @@ tracker_ontology_service_get_key_metadata (const gchar *service_str,
 	return 0;
 }
 
+gint
+tracker_ontology_service_get_key_collate (const gchar *service_str,
+					  const gchar *meta_name)
+{
+	gint		i;
+
+	g_return_val_if_fail (service_str != NULL, 0);
+	g_return_val_if_fail (meta_name != NULL, 0);
+
+	i = tracker_ontology_service_get_key_metadata (service_str, meta_name);
+
+	return ((i < 6) ? i : 0);
+}
+
 gboolean
 tracker_ontology_service_get_show_directories (const gchar *service_str)
 {
