@@ -170,6 +170,7 @@ typedef struct {
 	GString             *sql_group;
 	GString		    *sql_order;
 	gchar		    *service;
+	gboolean             multivalued_select;
 } ParserData;
 
 static void start_element_handler (GMarkupParseContext	*context,
@@ -393,6 +394,7 @@ add_metadata_field (ParserData	*data,
 				if (!tracker_field_data_get_is_select (field_data)) {
 					tracker_field_data_set_is_select (field_data, TRUE);
 					if(tracker_field_data_get_multiple_values(field_data)) {
+						data->multivalued_select = TRUE;
 						g_string_append_printf (data->sql_select, ", GROUP_CONCAT (%s)",
 									tracker_field_data_get_select_field (field_data));
 					} else {
@@ -421,6 +423,7 @@ add_metadata_field (ParserData	*data,
 			data->fields = g_slist_prepend (data->fields, field_data);
 			if (is_select) {
 				if(tracker_field_data_get_multiple_values(field_data)) {
+					data->multivalued_select = TRUE;
 					g_string_append_printf (data->sql_select, ", GROUP_CONCAT (%s)",
 								tracker_field_data_get_select_field (field_data));
 				} else {
@@ -742,22 +745,19 @@ start_element_handler (GMarkupParseContext  *context,
 	}
 }
 
-static gchar *
-get_value (const gchar *value, gboolean quote)
+static void
+replace_wildcards (gchar *str)
 {
-	gchar *escaped;
-	gchar *ret;
+	gchar *c;
 
-	escaped = tracker_escape_string(value);
-	if (quote) {
-		ret = g_strconcat (" '", escaped, "' ", NULL);
-	} else {
-		ret = g_strdup (escaped);
+	/* Replace glob-like wildcards
+	 * with % so we can use LIKE clauses
+	 */
+	for (c = str; *c; c++) {
+		if (*c == '*') {
+			*c = '%';
+		}
 	}
-
-	g_free (escaped);
-
-	return ret;
 }
 
 static gboolean
@@ -765,7 +765,7 @@ build_sql (ParserData *data)
 {
 	TrackerFieldData  *field_data;
 	ParseState	   state;
-	gchar		  *avalue, *value, *sub;
+	gchar		  *avalue, *value, *sub, *val;
 	const gchar	  *where_field;
 	GString		  *str;
 	gchar		 **s;
@@ -781,9 +781,11 @@ build_sql (ParserData *data)
 
 	state = peek_state (data);
 
-	avalue = get_value (data->current_value, (state != STATE_END_DATE &&
-						  state != STATE_END_INTEGER &&
-						  state != STATE_END_FLOAT));
+	avalue = tracker_escape_db_string (data->current_value, 
+					   (state != STATE_END_DATE &&
+					    state != STATE_END_INTEGER &&
+					    state != STATE_END_FLOAT),
+					   FALSE);
 
 	field_data = add_metadata_field (data, data->current_field, FALSE, TRUE, FALSE);
 
@@ -826,19 +828,22 @@ build_sql (ParserData *data)
 	case OP_EQUALS:
 		sub = strchr (data->current_value, '*');
 		if (sub) {
-			g_string_append_printf (str, " (%s glob '%s') ",
-						where_field,
-						data->current_value);
+			val = tracker_escape_db_string (data->current_value, FALSE, TRUE);
+			replace_wildcards (val);
+
+			g_string_append_printf (str, " (%s like '%s') ",
+						where_field, val);
+			g_free (val);
 		} else {
-			if ( !strlen(value) || (strcmp(value, " '' ") == 0) ) {
+			/* FIXME This check is too fragile */
+			if ( !strlen(value) || (strcmp(value, "''") == 0) ) {
 				tracker_field_data_set_needs_null (field_data, TRUE);
 				g_string_append_printf (str, " ((%s = '') OR %s IS NULL) ",
 							where_field,
 							where_field);
 			} else {
 				g_string_append_printf (str, " (%s = %s) ",
-							where_field,
-							value);
+							where_field, value);
 			}
 		}
 		break;
@@ -868,31 +873,35 @@ build_sql (ParserData *data)
 		break;
 
 	case OP_CONTAINS:
+		val = tracker_escape_db_string (data->current_value, FALSE, TRUE);
 		sub = strchr (data->current_value, '*');
 
 		if (sub) {
+			replace_wildcards (val);
 			g_string_append_printf (str, " (%s like '%%%%%s%%%%') ",
-						where_field,
-						data->current_value);
+						where_field, val);
 		} else {
 			g_string_append_printf (str, " (%s like '%%%%%s%%%%') ",
-						where_field,
-						data->current_value);
+						where_field, val);
 		}
+
+		g_free (val);
 		break;
 
 	case OP_STARTS:
+		val = tracker_escape_db_string (data->current_value, FALSE, TRUE);
 		sub = strchr (data->current_value, '*');
 
 		if (sub) {
+			replace_wildcards (val);
 			g_string_append_printf (str, " (%s like '%s') ",
-						where_field,
-						data->current_value);
+						where_field, val);
 		} else {
 			g_string_append_printf (str, " (%s like '%s%%%%') ",
-						where_field,
-						data->current_value);
+						where_field, val);
 		}
+
+		g_free (val);
 		break;
 
 	case OP_REGEX:
@@ -917,6 +926,8 @@ build_sql (ParserData *data)
 
 			g_string_append_printf (str, ") ) " );
 		}
+		g_strfreev(s);
+		
 		break;
 
 	default:
@@ -1109,8 +1120,7 @@ text_handler (GMarkupParseContext *context,
         case STATE_STRING:
         case STATE_DATE:
         case STATE_FLOAT:
-
-                data->current_value = g_strstrip (g_strndup (text, text_len));
+		data->current_value = g_strstrip (g_strndup (text, text_len));
                 break;
         default:
                 break;
@@ -1145,11 +1155,11 @@ get_select_header (const char *service)
         case 6:
         case 7:
         case 8:
-                g_string_append_printf (result, " Select DISTINCT (S.Path || '%s' || S.Name) as uri, GetServiceName(S.ServiceTypeID) as stype ", G_DIR_SEPARATOR_S);
+                g_string_append_printf (result, " Select (S.Path || '%s' || S.Name) as uri, GetServiceName(S.ServiceTypeID) as stype ", G_DIR_SEPARATOR_S);
                 break;
 
         default :
-                g_string_append_printf (result, " Select DISTINCT (S.Path || '%s' || S.Name) as uri, GetServiceName(S.ServiceTypeID) as stype ", G_DIR_SEPARATOR_S);
+                g_string_append_printf (result, " Select (S.Path || '%s' || S.Name) as uri, GetServiceName(S.ServiceTypeID) as stype ", G_DIR_SEPARATOR_S);
                 break;
 	}
 
@@ -1178,6 +1188,9 @@ append_where_header (GString *string, const char *service)
 		g_string_append_printf (string, " (S.ServiceTypeID=%d) ", type);
 	}
 
+	/* only search for enabled items */
+	g_string_append_printf (string, "AND (S.Enabled = 1) ");
+
 	/* only search for items on enabled volumes */
 	g_string_append_printf (string, "AND (S.AuxilaryID = 0 OR S.AuxilaryID IN (SELECT VolumeID FROM Volumes WHERE Enabled = 1)) ");
 }
@@ -1221,6 +1234,9 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 	data.iface = iface;
 	data.statement_count = 0;
 	data.service = (gchar*) service;
+
+	data.multivalued_select = FALSE;
+
 	data.sql_select = get_select_header (service);
 
 	if (field_count > 0) {
@@ -1296,7 +1312,7 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 			}
 
 			list = g_hash_table_lookup (table, key);
-			list = g_list_prepend (list, tracker_escape_string(value));
+			list = g_list_prepend (list, tracker_escape_db_string (value, TRUE, FALSE));
 			g_hash_table_insert (table, g_strdup (key), list);
 
 			g_free (full);
@@ -1312,7 +1328,7 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 
 			for (l = list; l; l = l->next) {
 				g_string_append_printf (data.sql_where,
-				     " AND (S.ID IN (SELECT ServiceID FROM ServiceKeywordMetaData WHERE MetaDataID in (%s) AND MetadataValue = '%s')) ",
+							" AND (S.ID IN (SELECT ServiceID FROM ServiceKeywordMetaData WHERE MetaDataID in (%s) AND MetadataValue = %s)) ",
 							keyword_metadata,
 							(gchar*) l->data);
 			}
@@ -1329,7 +1345,7 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 		g_string_append_printf (data.sql_where, "AND ");
 	}
 
-	data.sql_group = g_string_new (" GROUP BY S.ID ");
+	data.sql_group = g_string_new ("");
 
 	if (limit < 1) {
 		limit = 1024;
@@ -1357,8 +1373,22 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 
 		for (i = 0; i < sort_field_count; i++) {
 			TrackerFieldData *field_data;
+			gchar           **s;
+			gboolean          desc;
 
-			field_data = add_metadata_field (&data, sort_fields[i], FALSE, FALSE, TRUE);
+			s = g_strsplit (sort_fields[i], " ", 2);
+
+			field_data = add_metadata_field (&data, s[0], FALSE, FALSE, TRUE);
+
+			if (s[1]) {
+				if ((strcmp (s[1], "ASC") == 0) || (strcmp (s[1], "A")==0)) {
+					desc = FALSE;
+				} else {
+					desc = TRUE;
+				}
+			} else {
+				desc = sort_desc;
+			}
 
 			if (!field_data) {
 				g_set_error (error,
@@ -1384,7 +1414,8 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 
 			g_string_append_printf (data.sql_order, "%s %s",
 						tracker_field_data_get_order_field (field_data),
-						sort_desc ? "DESC" : "ASC");
+						desc ? "DESC" : "ASC");
+			g_strfreev(s);
 		}
 	}
 
@@ -1468,6 +1499,10 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 			}
 		}
 
+		if (data.multivalued_select) {
+			g_string_append_printf (data.sql_group, " GROUP BY S.ID ");
+		}
+
 		result = g_strconcat (data.sql_select->str,
 				      " ",
 				      data.sql_from->str,
@@ -1506,32 +1541,35 @@ tracker_rdf_query_to_sql (TrackerDBInterface  *iface,
 }
 
 
-/*
- * The following function turns an rdf query into a filter that can be used for example
- * for getting unique values of a field with a certain query.
- * FIXME It is not pretty. The calling function needs to define Services S that is joined in this method.
+/* The following function turns an rdf query into a filter that can be
+ * used for example for getting unique values of a field with a
+ * certain query. FIXME It is not pretty. The calling function needs
+ * to define Services S that is joined in this method.
  */
-
 void
 tracker_rdf_filter_to_sql (TrackerDBInterface *iface,
-			   const char	      *query,
-			   const char	      *service,
+			   const gchar	      *query,
+			   const gchar	      *service,
 			   GSList	     **fields,
-			   char		     **from,
-			   char		     **where,
+			   gchar	     **from,
+			   gchar	     **where,
 			   GError	     **error)
 {
 	ParserData data;
 
 	g_return_if_fail (TRACKER_IS_DB_INTERFACE (iface));
+	g_return_if_fail (query != NULL);
 	g_return_if_fail (service != NULL);
 	g_return_if_fail (from != NULL);
 	g_return_if_fail (where != NULL);
+	g_return_if_fail (fields != NULL);
 
 	memset (&data, 0, sizeof (data));
 	data.iface = iface;
 	data.statement_count = 0;
-	data.service = (char *) service;
+	data.service = (gchar *) service;
+
+	data.multivalued_select = FALSE;
 
 	data.sql_from = g_string_new ("");
 	data.sql_where = g_string_new ("");
@@ -1557,14 +1595,9 @@ tracker_rdf_filter_to_sql (TrackerDBInterface *iface,
 
 	push_stack (&data, STATE_START);
 
-	if ( (query != NULL) && (!g_markup_parse_context_parse (data.context, query, -1, error))) {
-
+	if (!g_markup_parse_context_parse (data.context, query, -1, error)) {
 		*from = NULL;
 		*where = NULL;
-
-		g_string_free (data.sql_from, TRUE);
-		g_string_free (data.sql_where, TRUE);
-
 	} else {
 		GSList *l;
 
@@ -1616,28 +1649,20 @@ tracker_rdf_filter_to_sql (TrackerDBInterface *iface,
 			}
 		}
 
-
 		*from  = g_strdup (data.sql_from->str);
 		*where = g_strdup (data.sql_where->str);
-		g_string_free (data.sql_from, TRUE);
-		g_string_free (data.sql_where, TRUE);
-
-
 	}
 
+	g_string_free (data.sql_from, TRUE);
+	g_string_free (data.sql_where, TRUE);
+
 	*fields = data.fields;
 
 	g_slist_free (data.stack);
 	g_markup_parse_context_free (data.context);
 
-	if (data.current_field) {
-		g_free (data.current_field);
-	}
-
-	if (data.current_value) {
-		g_free (data.current_value);
-	}
-
+	g_free (data.current_field);
+	g_free (data.current_value);
 	g_free (data.parser);
 }
 
