@@ -56,7 +56,6 @@
 #include <libtracker-common/tracker-config.h>
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-hal.h>
 #include <libtracker-common/tracker-language.h>
 #include <libtracker-common/tracker-parser.h>
 #include <libtracker-common/tracker-ontology.h>
@@ -96,12 +95,12 @@
 #define TRACKER_INDEXER_ERROR_CODE  0
 
 /* Properties that change in move event */
-#define METADATA_FILE_NAME_DELIMITED "File:NameDelimited"
-#define METADATA_FILE_EXT	     "File:Ext"
 #define METADATA_FILE_PATH	     "File:Path"
 #define METADATA_FILE_NAME	     "File:Name"
 #define METADATA_FILE_MIMETYPE       "File:Mime"
 
+#undef ENABLE_TTL_LOADER
+
 typedef struct PathInfo PathInfo;
 typedef struct MetadataForeachData MetadataForeachData;
 typedef struct MetadataRequest MetadataRequest;
@@ -117,6 +116,7 @@ struct TrackerIndexerPrivate {
 	GQuark current_module;
 	GHashTable *indexer_modules;
 
+	gchar *db_mode;
 	gchar *db_dir;
 
 	TrackerDBIndex *file_index;
@@ -166,9 +166,12 @@ struct PathInfo {
 };
 
 struct MetadataForeachData {
+	TrackerDataUpdateMetadataContext *context;
 	TrackerLanguage *language;
 	TrackerConfig *config;
+	guint32 service_id;
 	TrackerService *service;
+	TrackerDBIndex *index;
 	gboolean add;
 	guint32 id;
 };
@@ -180,9 +183,10 @@ struct UpdateWordsForeachData {
 
 enum TrackerIndexerState {
 	TRACKER_INDEXER_STATE_INDEX_OVERLOADED = 1 << 0,
-	TRACKER_INDEXER_STATE_PAUSED	= 1 << 1,
-	TRACKER_INDEXER_STATE_STOPPED	= 1 << 2,
-	TRACKER_INDEXER_STATE_CLEANUP   = 1 << 3
+	TRACKER_INDEXER_STATE_INDEX_ERROR = 1 << 1,
+	TRACKER_INDEXER_STATE_PAUSED	= 1 << 2,
+	TRACKER_INDEXER_STATE_STOPPED	= 1 << 3,
+	TRACKER_INDEXER_STATE_CLEANUP   = 1 << 4
 };
 
 enum {
@@ -436,7 +440,6 @@ tracker_indexer_transaction_commit (TrackerIndexer *indexer)
 {
 	stop_transaction (indexer);
 	tracker_indexer_set_running (indexer, TRUE);
-
 }
 
 void
@@ -446,6 +449,14 @@ tracker_indexer_transaction_open (TrackerIndexer *indexer)
 	start_transaction (indexer);
 }
 
+TrackerHal *
+tracker_indexer_get_hal (TrackerIndexer *indexer)
+{
+	g_return_val_if_fail (TRACKER_IS_INDEXER (indexer), NULL);
+
+	return indexer->private->hal;
+}
+
 #ifdef HAVE_HAL
 
 static void
@@ -539,6 +550,7 @@ index_error_received_cb (TrackerDBIndex *index,
 			 const GError   *error,
 			 TrackerIndexer *indexer)
 {
+	state_set_flags (indexer, TRACKER_INDEXER_STATE_INDEX_ERROR);
 	g_signal_emit (indexer, signals[INDEXING_ERROR], 0,
 		       error->message, TRUE);
 }
@@ -616,6 +628,8 @@ tracker_indexer_finalize (GObject *object)
 	 */
 	stop_scheduled_flush (TRACKER_INDEXER (object));
 
+	g_free (priv->db_mode);
+
 	if (priv->pause_for_duration_id) {
 		g_source_remove (priv->pause_for_duration_id);
 	}
@@ -695,10 +709,6 @@ tracker_indexer_get_property (GObject	 *object,
 			      GValue	 *value,
 			      GParamSpec *pspec)
 {
-	TrackerIndexerPrivate *priv;
-
-	priv = TRACKER_INDEXER_GET_PRIVATE (object);
-
 	switch (prop_id) {
 	case PROP_RUNNING:
 		g_value_set_boolean (value,
@@ -1017,6 +1027,26 @@ tracker_indexer_load_modules (TrackerIndexer *indexer)
 	g_list_free (modules);
 }
 
+static void
+set_up_databases (TrackerIndexer *indexer)
+{
+	TrackerIndexerPrivate *priv;
+
+	priv = indexer->private;
+
+	/* Set up databases, these pointers are mostly used to
+	 * start/stop transactions, since TrackerDBManager treats
+	 * interfaces as singletons, it's safe to just ask it
+	 * again for an interface.
+	 */
+	priv->cache = tracker_db_manager_get_db_interface (TRACKER_DB_CACHE);
+	priv->common = tracker_db_manager_get_db_interface (TRACKER_DB_COMMON);
+	priv->file_metadata = tracker_db_manager_get_db_interface (TRACKER_DB_FILE_METADATA);
+	priv->file_contents = tracker_db_manager_get_db_interface (TRACKER_DB_FILE_CONTENTS);
+	priv->email_metadata = tracker_db_manager_get_db_interface (TRACKER_DB_EMAIL_METADATA);
+	priv->email_contents = tracker_db_manager_get_db_interface (TRACKER_DB_EMAIL_CONTENTS);
+}
+
 static void
 tracker_indexer_init (TrackerIndexer *indexer)
 {
@@ -1081,17 +1111,7 @@ tracker_indexer_init (TrackerIndexer *indexer)
 	g_signal_connect (priv->email_index, "error-received",
 			  G_CALLBACK (index_error_received_cb), indexer);
 
-	/* Set up databases, these pointers are mostly used to
-	 * start/stop transactions, since TrackerDBManager treats
-	 * interfaces as singletons, it's safe to just ask it
-	 * again for an interface.
-	 */
-	priv->cache = tracker_db_manager_get_db_interface (TRACKER_DB_CACHE);
-	priv->common = tracker_db_manager_get_db_interface (TRACKER_DB_COMMON);
-	priv->file_metadata = tracker_db_manager_get_db_interface (TRACKER_DB_FILE_METADATA);
-	priv->file_contents = tracker_db_manager_get_db_interface (TRACKER_DB_FILE_CONTENTS);
-	priv->email_metadata = tracker_db_manager_get_db_interface (TRACKER_DB_EMAIL_METADATA);
-	priv->email_contents = tracker_db_manager_get_db_interface (TRACKER_DB_EMAIL_CONTENTS);
+	set_up_databases (indexer);
 
 	/* Set up volume monitor */
 	priv->volume_monitor = g_volume_monitor_get ();
@@ -1127,13 +1147,12 @@ index_metadata_item (TrackerField	 *field,
 		     const gchar	 *value,
 		     MetadataForeachData *data)
 {
-	TrackerDBIndex *lindex;
+	gchar *word, *end;
 	gchar *parsed_value;
-	gchar **arr;
-	gint service_id;
-	gint i;
 	gint score;
+	gboolean reached_end = FALSE;
 
+#ifdef ENABLE_FTS
 	parsed_value = tracker_parser_text_to_string (value,
 						      data->language,
 						      tracker_config_get_max_word_length (data->config),
@@ -1141,7 +1160,25 @@ index_metadata_item (TrackerField	 *field,
 						      tracker_field_get_filtered (field),
 						      tracker_field_get_filtered (field),
 						      tracker_field_get_delimited (field));
+#else
+	parsed_value = NULL;
+#endif /* ENABLE_FTS */
 
+	if (data->add) {
+		tracker_data_update_set_metadata (data->context,
+						  data->service,
+						  data->id,
+						  field,
+						  value,
+						  parsed_value);
+	} else {
+		tracker_data_update_delete_metadata (data->service,
+						     data->id,
+						     field,
+						     value);
+	}
+
+#ifdef ENABLE_FTS
 	if (!parsed_value) {
 		return;
 	}
@@ -1152,26 +1189,48 @@ index_metadata_item (TrackerField	 *field,
 		score = -1 * tracker_field_get_weight (field);
 	}
 
-	arr = g_strsplit (parsed_value, " ", -1);
-	service_id = tracker_service_get_id (data->service);
-	lindex = tracker_db_index_manager_get_index_by_service_id (service_id);
+	word = parsed_value;
 
-	for (i = 0; arr[i]; i++) {
-		tracker_db_index_add_word (lindex,
-					   arr[i],
+	while (word && *word) {
+		/* Skip whitespaces */
+		while (*word == ' ') {
+			word++;
+		}
+
+		if (!*word) {
+			/* Reached end of the string */
+			break;
+		}
+
+		/* Find end of word */
+		end = word;
+
+		while (*end != ' ' && *end != '\0') {
+			end++;
+		}
+
+		if (!*end) {
+			/* Reached end of the string */
+			reached_end = TRUE;
+		}
+
+		*end = '\0';
+
+		tracker_db_index_add_word (data->index,
+					   word,
 					   data->id,
-					   tracker_service_get_id (data->service),
+					   data->service_id,
 					   score);
+
+		if (!reached_end) {
+			word = end + 1;
+		} else {
+			word = NULL;
+		}
 	}
 
-	if (data->add) {
-		tracker_data_update_set_metadata (data->service, data->id, field, (gchar *) value, parsed_value);
-	} else {
-		tracker_data_update_delete_metadata (data->service, data->id, field, (gchar *)value);
-	}
-
-	g_strfreev (arr);
 	g_free (parsed_value);
+#endif
 }
 
 static void
@@ -1210,19 +1269,27 @@ index_metadata_foreach (TrackerField *field,
 
 static void
 index_metadata (TrackerIndexer	      *indexer,
+		TrackerDataUpdateMetadataContext *context,
 		guint32		       id,
 		TrackerService	      *service,
 		TrackerModuleMetadata *metadata)
 {
 	MetadataForeachData data;
+	gint service_id;
 
+	service_id = tracker_service_get_id (service);
+
+	data.context = context;
 	data.language = indexer->private->language;
 	data.config = indexer->private->config;
+	data.service_id = service_id;
 	data.service = service;
+	data.index = tracker_db_index_manager_get_index_by_service_id (service_id);
 	data.id = id;
 	data.add = TRUE;
 
-	tracker_module_metadata_foreach (metadata, index_metadata_foreach, &data);
+	tracker_data_metadata_foreach (TRACKER_DATA_METADATA (metadata),
+				       index_metadata_foreach, &data);
 
 	schedule_flush (indexer, FALSE);
 }
@@ -1234,10 +1301,16 @@ unindex_metadata (TrackerIndexer      *indexer,
 		  TrackerDataMetadata *metadata)
 {
 	MetadataForeachData data;
+	gint service_id;
 
+	service_id = tracker_service_get_id (service);
+
+	data.context = NULL;
 	data.language = indexer->private->language;
 	data.config = indexer->private->config;
+	data.service_id = service_id;
 	data.service = service;
+	data.index = tracker_db_index_manager_get_index_by_service_id (service_id);
 	data.id = id;
 	data.add = FALSE;
 
@@ -1541,14 +1614,14 @@ remove_stale_children (TrackerIndexer *indexer,
 	TrackerDBInterface *iface;
 	gchar **children;
 	PathInfo *info;
-	gint i;
+	gint found, i;
 
 	iface = tracker_db_manager_get_db_interface_by_type (tracker_service_get_name (service),
 							     TRACKER_DB_CONTENT_TYPE_METADATA);
 
 	children = tracker_data_search_files_get (iface, path);
 
-	for (i = 0; children[i]; i++) {
+	for (i = 0, found = 0; children[i]; i++) {
 		GFile *file;
 
 		file = g_file_new_for_path (children[i]);
@@ -1557,11 +1630,16 @@ remove_stale_children (TrackerIndexer *indexer,
 			/* File doesn't exist, check for deletion */
 			info = path_info_new (parent_info->module, file, NULL, TRUE);
 			add_file (indexer, info);
+
+			found++;
 		}
 
 		g_object_unref (file);
 	}
 
+	g_message ("'%s' checked for stale children, found:%d", 
+		   path, found);
+
 	g_strfreev (children);
 }
 
@@ -1571,22 +1649,27 @@ generate_item_thumbnail (TrackerIndexer        *indexer,
 			 const gchar           *basename,
 			 TrackerModuleMetadata *metadata)
 {
-	const gchar *path, *mime_type;
+	const gchar *mime_type;
 
-	path = tracker_module_metadata_lookup (metadata, METADATA_FILE_NAME_DELIMITED, FALSE);
-	mime_type = tracker_module_metadata_lookup (metadata, METADATA_FILE_MIMETYPE, FALSE);
+	mime_type = tracker_data_metadata_lookup (TRACKER_DATA_METADATA (metadata),
+						  METADATA_FILE_MIMETYPE);
 
-	if (path && path[0] == G_DIR_SEPARATOR && mime_type &&
+	if (dirname && 
+	    dirname[0] == G_DIR_SEPARATOR && 
+	    mime_type &&
 	    tracker_config_get_enable_thumbnails (indexer->private->config)) {
 		GFile *file;
 		gchar *uri;
+		gchar *path;
 
+		path = g_build_filename (dirname, basename, NULL);
 		file = g_file_new_for_path (path);
+		g_free (path);
+
 		uri = g_file_get_uri (file);
+		g_object_unref (file);
 
 		tracker_thumbnailer_queue_file (uri, mime_type);
-
-		g_object_unref (file);
 		g_free (uri);
 	}
 }
@@ -1611,6 +1694,15 @@ item_add_or_update (TrackerIndexer        *indexer,
 		return;
 	}
 
+	if (G_UNLIKELY (!indexer->private->in_transaction)) {
+		start_transaction (indexer);
+	}
+
+	service_path = g_build_path (G_DIR_SEPARATOR_S,
+				     dirname,
+				     basename,
+				     NULL);
+
 	exists = tracker_data_query_service_exists (service, dirname, basename, &id, NULL, &enabled);
 
 	if (exists && !enabled) {
@@ -1620,21 +1712,19 @@ item_add_or_update (TrackerIndexer        *indexer,
 	}
 
 	if (exists) {
+		TrackerDataUpdateMetadataContext *context;
 		TrackerDataMetadata *old_metadata_emb, *old_metadata_non_emb;
 		gchar *old_text;
 
 		if (tracker_module_file_get_flags (info->module_file) & TRACKER_FILE_CONTENTS_STATIC) {
 			/* According to the module, the metadata can't change for this item */
-			g_debug ("Not updating static item '%s/%s'",
-				 dirname,
-				 basename);
+			g_debug ("Not updating static item '%s'", service_path);
+			g_free (service_path);
 			return;
 		}
 
 		/* Update case */
-		g_debug ("Updating item '%s/%s'", 
-			 dirname, 
-			 basename);
+		g_debug ("Updating item '%s'", service_path);
 
 		/* "metadata" (new metadata) contains embedded props and can contain
 		 * non-embedded properties with default values! Dont overwrite those 
@@ -1650,11 +1740,17 @@ item_add_or_update (TrackerIndexer        *indexer,
 
 		unindex_metadata (indexer, id, service, old_metadata_emb);
 
-		tracker_module_metadata_foreach_remove (metadata,
-							remove_existing_non_emb_metadata,
-							old_metadata_non_emb);
+		tracker_data_metadata_foreach_remove (TRACKER_DATA_METADATA (metadata),
+						      remove_existing_non_emb_metadata,
+						      old_metadata_non_emb);
 
-		index_metadata (indexer, id, service, metadata);
+		context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_UPDATE,
+								    service, id);
+
+		index_metadata (indexer, context, id, service, metadata);
+
+		tracker_data_update_metadata_context_close (context);
+		tracker_data_update_metadata_context_free (context);
 
 		/* Take the old text -> the new one, calculate
 		 * difference and add the words.
@@ -1664,39 +1760,46 @@ item_add_or_update (TrackerIndexer        *indexer,
 		item_update_content (indexer, service, id, old_text, text);
 
 		if (strcmp (tracker_service_get_name (service), "Folders") == 0) {
-			gchar *path;
-
 			/* Remove no longer existing children, this is necessary in case
 			 * there were files added/removed in a directory between tracker
 			 * executions
 			 */
-
-			path = g_build_path (G_DIR_SEPARATOR_S, dirname, basename, NULL);
-			remove_stale_children (indexer, service, info, path);
-			g_free (path);
+			remove_stale_children (indexer, service, info, service_path);
 		}
 
 		g_free (old_text);
-		tracker_data_metadata_free (old_metadata_emb);
-		tracker_data_metadata_free (old_metadata_non_emb);
+		g_object_unref (old_metadata_emb);
+		g_object_unref (old_metadata_non_emb);
 	} else {
+		TrackerDataUpdateMetadataContext *context;
 		GHashTable *data;
+		const gchar *udi;
 
-		g_debug ("Adding item '%s/%s'",
-			 dirname,
-			 basename);
+		g_debug ("Adding item '%s'", service_path);
 
 		/* Service wasn't previously indexed */
 		id = tracker_data_update_get_new_service_id (indexer->private->common);
 		data = tracker_module_metadata_get_hash_table (metadata);
+#ifdef HAVE_HAL
+		udi = tracker_hal_udi_get_for_path (indexer->private->hal, service_path);
+#else
+		udi = NULL;
+#endif
+		context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_INSERT,
+								    service, id);
 
-		tracker_data_update_create_service (service,
+		tracker_data_update_create_service (context,
+						    service,
 						    id,
+						    udi,
 						    dirname,
 						    basename,
 						    data);
 
-		index_metadata (indexer, id, service, metadata);
+		index_metadata (indexer, context, id, service, metadata);
+
+		tracker_data_update_metadata_context_close (context);
+		tracker_data_update_metadata_context_free (context);
 
 		if (text) {
 			/* Save in the index */
@@ -1715,26 +1818,21 @@ item_add_or_update (TrackerIndexer        *indexer,
 
 	generate_item_thumbnail (indexer, dirname, basename, metadata);
 
-	/* TODO: URI branch path -> uri */
-
-	service_path = g_build_path (G_DIR_SEPARATOR_S, 
-				     dirname, 
-				     basename, 
-				     NULL);
-
+#ifdef ENABLE_TTL_LOADER
 #ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     service_path, 
 						     &mount_point,
 						     NULL)) {
-
 		tracker_removable_device_add_metadata (indexer, 
 						       mount_point, 
 						       service_path, 
 						       tracker_service_get_name (service),
 						       metadata);
 	}
-#endif
+#endif /* HAVE_HAL */
+#endif /* ENABLE_TTL_LOADER */
+
 	g_free (mount_point);
 	g_free (service_path);
 }
@@ -1749,13 +1847,12 @@ filter_invalid_after_move_properties (TrackerField *field,
 
 	name = tracker_field_get_name (field);
 
-	if (g_strcmp0 (name, METADATA_FILE_NAME_DELIMITED) == 0 ||
-	    g_strcmp0 (name, METADATA_FILE_NAME) == 0 ||
-	    g_strcmp0 (name, METADATA_FILE_PATH) == 0 ||
-	    g_strcmp0 (name, METADATA_FILE_EXT) == 0) {
+	if (g_strcmp0 (name, METADATA_FILE_NAME) == 0 ||
+	    g_strcmp0 (name, METADATA_FILE_PATH) == 0) {
 		return FALSE;
 	}
 
+
 	return TRUE;
 }
 
@@ -1795,6 +1892,8 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 				    GFile          *file,
 				    GFile          *source_file)
 {
+#ifdef ENABLE_TTL_LOADER
+#ifdef HAVE_HAL
 	const gchar *service_name;
 	gchar *path, *source_path;
 	gchar *mount_point = NULL;
@@ -1803,23 +1902,19 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 	path = g_file_get_path (file);
 	source_path = g_file_get_path (source_file);
 
-#ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     source_path,
 						     &mount_point,
-						     NULL) ) {
-
+						     NULL)) {
 		if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
-						     path,
-						     NULL,
-						     NULL) ) {
-
+							     path,
+							     NULL,
+							     NULL)) {
 			tracker_removable_device_add_move (indexer,
 							   mount_point,
 							   source_path,
 							   path,
 							   service_name);
-
 		} else {
 			tracker_removable_device_add_removal (indexer,
 							      mount_point,
@@ -1827,10 +1922,12 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 							      service_name);
 		}
 	}
-#endif
+
 	g_free (mount_point);
 	g_free (source_path);
 	g_free (path);
+#endif /* HAVE_HAL */
+#endif /* ENABLE_TTL_LOADER */
 }
 
 static void
@@ -1841,9 +1938,9 @@ update_moved_item_index (TrackerIndexer      *indexer,
 			 GFile               *file,
 			 GFile               *source_file)
 {
+	TrackerDataUpdateMetadataContext *context;
 	TrackerModuleMetadata *new_metadata;
 	gchar *path, *new_path, *new_name;
-	const gchar *ext;
 
 	path = g_file_get_path (file);
 
@@ -1862,15 +1959,14 @@ update_moved_item_index (TrackerIndexer      *indexer,
 
 	tracker_module_metadata_add_string (new_metadata, METADATA_FILE_PATH, new_path);
 	tracker_module_metadata_add_string (new_metadata, METADATA_FILE_NAME, new_name);
-	tracker_module_metadata_add_string (new_metadata, METADATA_FILE_NAME_DELIMITED, path);
 
-	ext = strrchr (path, '.');
-	if (ext) {
-		ext++;
-		tracker_module_metadata_add_string (new_metadata, METADATA_FILE_EXT, ext);
-	}
+	context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_UPDATE,
+							    service, service_id);
 
-	index_metadata (indexer, service_id, service, new_metadata);
+	index_metadata (indexer, context, service_id, service, new_metadata);
+
+	tracker_data_update_metadata_context_close (context);
+	tracker_data_update_metadata_context_free (context);
 
 	g_object_unref (new_metadata);
 	g_free (new_path);
@@ -1893,12 +1989,16 @@ item_erase (TrackerIndexer *indexer,
 	data_metadata = tracker_data_query_metadata (service, service_id, TRUE);
 
 	if (data_metadata) {
-		const gchar *path, *mime_type;
+		const gchar *mime_type;
+		gchar *path;
 		GFile *file;
 		gchar *uri;
 
 		/* TODO URI branch: this is a URI conversion */
-		path = tracker_data_metadata_lookup (data_metadata, "File:NameDelimited");
+		path = g_build_path (G_DIR_SEPARATOR_S,
+		                     tracker_data_metadata_lookup (data_metadata, "File:Path"),
+				     tracker_data_metadata_lookup (data_metadata, "File:Name"),
+				     NULL);
 		file = g_file_new_for_path (path);
 		uri = g_file_get_uri (file);
 		g_object_unref (file);
@@ -1906,8 +2006,9 @@ item_erase (TrackerIndexer *indexer,
 		mime_type = tracker_data_metadata_lookup (data_metadata, "File:Mime");
 		tracker_thumbnailer_remove (uri, mime_type);
 
-		tracker_data_metadata_free (data_metadata);
+		g_object_unref (data_metadata);
 		g_free (uri);
+		g_free (path);
 	}
 
 	/* Get content, unindex the words and delete the contents */
@@ -2031,7 +2132,7 @@ item_move (TrackerIndexer  *indexer,
 		g_free (source_path);
 
 		if (old_metadata) {
-			tracker_data_metadata_free (old_metadata);
+			g_object_unref (old_metadata);
 		}
 
 		return FALSE;
@@ -2070,7 +2171,7 @@ item_move (TrackerIndexer  *indexer,
 	}
 
 	if (old_metadata) {
-		tracker_data_metadata_free (old_metadata);
+		g_object_unref (old_metadata);
 	}
 
 	g_free (source_path);
@@ -2098,6 +2199,10 @@ item_mark_for_removal (TrackerIndexer *indexer,
 		 dirname, 
 		 basename);
 
+	if (G_UNLIKELY (!indexer->private->in_transaction)) {
+		start_transaction (indexer);
+	}
+
 	/* The file is not anymore in the filesystem. Obtain
 	 * the service type from the DB.
 	 */
@@ -2152,17 +2257,20 @@ item_mark_for_removal (TrackerIndexer *indexer,
 
 		g_hash_table_destroy (children);
 	}
+
+#ifdef ENABLE_TTL_LOADER
 #ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     path, 
 						     &mount_point,
 						     NULL)) {
-
 		tracker_removable_device_add_removal (indexer, mount_point, 
 						      path,
 						      tracker_service_get_name (service));
 	}
-#endif
+#endif /* HAVE_HAL */
+#endif /* ENABLE_TTL_LOADER */
+
 	g_free (mount_point);
 	g_free (path);
 }
@@ -2174,7 +2282,6 @@ item_process (TrackerIndexer *indexer,
 	      const gchar    *basename)
 {
 	TrackerModuleMetadata *metadata;
-	gchar *text;
 
 	metadata = tracker_module_file_get_metadata (info->module_file);
 
@@ -2187,7 +2294,13 @@ item_process (TrackerIndexer *indexer,
 	}
 
 	if (metadata) {
+		gchar *text;
+
+#ifdef ENABLE_FTS
 		text = tracker_module_file_get_text (info->module_file);
+#else  /* ENABLE_FTS */
+		text = NULL;
+#endif /* ENABLE_FTS */
 
 		if (tracker_module_file_is_cancelled (info->module_file)) {
 			g_object_unref (metadata);
@@ -2219,6 +2332,7 @@ handle_metadata_add (TrackerIndexer *indexer,
 		     GStrv	     values,
 		     GError	   **error)
 {
+	TrackerDataUpdateMetadataContext *context;
 	TrackerService *service;
 	TrackerField   *field;
 	guint           service_id, i, j;
@@ -2313,6 +2427,9 @@ handle_metadata_add (TrackerIndexer *indexer,
 
 	set_values = g_new0 (gchar *, g_strv_length (values) + 1);
 
+	context = tracker_data_update_metadata_context_new (TRACKER_CONTEXT_TYPE_UPDATE,
+							    service, service_id);
+
 	for (i = 0, j = 0; values[i] != NULL; i++) {
 		g_debug ("Setting metadata: service_type '%s' id '%d' field '%s' value '%s'",
 			 tracker_service_get_name (service),
@@ -2320,15 +2437,23 @@ handle_metadata_add (TrackerIndexer *indexer,
 			 tracker_field_get_name (field),
 			 values[i]);
 
-		if (tracker_field_get_multiple_values (field) 
-		    && (tracker_string_in_string_list (values[i], old_contents) > -1) ) {
+		if (tracker_field_get_multiple_values (field) &&
+		    tracker_string_in_string_list (values[i], old_contents) > -1) {
 			continue;
 		}
 
-		tracker_data_update_set_metadata (service, service_id, field, values[i], NULL);
+		tracker_data_update_set_metadata (context, 
+						  service, 
+						  service_id, 
+						  field, 
+						  values[i], 
+						  NULL);
 		set_values [++j] = values[i];
 	}
 
+	tracker_data_update_metadata_context_close (context);
+	tracker_data_update_metadata_context_free (context);
+
 	joined = g_strjoinv (" ", set_values);
 	if (tracker_field_get_filtered (field)) {
 		index_text_no_parsing (indexer,
@@ -2504,6 +2629,11 @@ should_change_index_for_file (TrackerIndexer *indexer,
 		return TRUE;
 	}
 
+	/* If the service is a folder, we check it for stale children */
+	if (strcmp (tracker_service_get_name (service), "Folders") == 0) {
+		return TRUE;
+	}
+
 	/* So, if we are here, then the file or directory DID exist
 	 * in the database already. Now we need to check if the
 	 * parent directory mtime matches the mtime we have for it in
@@ -2935,12 +3065,16 @@ static gchar *
 state_to_string (TrackerIndexerState state)
 {
 	GString *s;
+	gchar   *str, *p;
 
 	s = g_string_new ("");
 	
 	if (state & TRACKER_INDEXER_STATE_INDEX_OVERLOADED) {
 		s = g_string_append (s, "INDEX_OVERLOADED | ");
 	}
+	if (state & TRACKER_INDEXER_STATE_INDEX_ERROR) {
+		s = g_string_append (s, "INDEX_ERROR | ");
+	}
 	if (state & TRACKER_INDEXER_STATE_PAUSED) {
 		s = g_string_append (s, "PAUSED | ");
 	}
@@ -2951,9 +3085,19 @@ state_to_string (TrackerIndexerState state)
 		s = g_string_append (s, "CLEANUP | ");
 	}
 
-	s->str[s->len - 3] = '\0';
+	str = g_string_free (s, FALSE);
 
-	return g_string_free (s, FALSE);
+	/* Remove last separator */
+	p = g_utf8_strrchr (str, -1, '|');
+	if (p) {
+		/* Go back one to the space before '|' */
+		p--;
+		
+		/* NULL terminate here */
+		*p = '\0';
+	}
+
+	return str;
 }
 
 static void
@@ -3251,6 +3395,65 @@ tracker_indexer_files_delete (TrackerIndexer *indexer,
 				     files, context, error);
 }
 
+void
+tracker_indexer_files_move (TrackerIndexer *indexer,
+			    const gchar *module_name,
+			    GStrv from_files,
+			    GStrv to_files,
+			    DBusGMethodInvocation *context,
+			    GError **error)
+{
+	TrackerIndexerModule *module;
+	guint request_id;
+	gint i;
+	GError *actual_error;
+
+	request_id = tracker_dbus_get_next_request_id ();
+
+	tracker_dbus_async_return_if_fail (TRACKER_IS_INDEXER (indexer), context);
+	tracker_dbus_async_return_if_fail (from_files != NULL, context);
+	tracker_dbus_async_return_if_fail (to_files != NULL, context);
+	tracker_dbus_async_return_if_fail (g_strv_length (from_files) == g_strv_length (to_files), context);
+
+	tracker_dbus_request_new (request_id,
+				  "DBus request to move %d files",
+				  g_strv_length (from_files));
+
+	module = g_hash_table_lookup (indexer->private->indexer_modules, module_name);
+
+	if (!module) {
+		tracker_dbus_request_failed (request_id,
+					     &actual_error,
+					     "The module '%s' is not loaded",
+					     module_name);
+		dbus_g_method_return_error (context, actual_error);
+		g_error_free (actual_error);
+		return;
+	}
+
+	for (i = 0; i < g_strv_length (from_files); i++) {
+		GFile *file_from, *file_to;
+		PathInfo *info;
+		const gchar *str_from, *str_to;
+
+		str_from = from_files[i];
+		str_to = to_files[i];
+
+		file_from = g_file_new_for_path (str_from);
+		file_to = g_file_new_for_path (str_to);
+		
+		/* Add files to the queue */
+		info = path_info_new (module, file_to, file_from, TRUE);
+		add_file (indexer, info);
+		
+		g_object_unref (file_from);
+		g_object_unref (file_to);
+	}
+
+	dbus_g_method_return (context);
+	tracker_dbus_request_success (request_id);
+}
+
 void
 tracker_indexer_file_move (TrackerIndexer	  *indexer,
 			   const gchar		  *module_name,
@@ -3356,9 +3559,11 @@ tracker_indexer_volume_update_state (TrackerIndexer         *indexer,
 	/* tracker_turtle_process_ttl will be spinning the mainloop, therefore
 	   we can already return the DBus method */
 
+#ifdef ENABLE_TTL_LOADER
 	if (enabled) {
 		tracker_removable_device_load (indexer, path);
 	}
+#endif /* ENABLE_TTL_LOADER */
 }
 
 void
@@ -3535,3 +3740,84 @@ tracker_indexer_shutdown (TrackerIndexer	 *indexer,
 	dbus_g_method_return (context);
 	tracker_dbus_request_success (request_id);
 }
+
+static gboolean
+set_profile (TrackerIndexer *indexer,
+	     const gchar    *profile_name)
+{
+	TrackerDBManagerFlags flags = TRACKER_DB_MANAGER_FORCE_NO_REINDEX;
+	TrackerIndexerPrivate *priv;
+	gboolean return_val = TRUE;
+
+	priv = indexer->private;
+
+	if (g_strcmp0 (profile_name, priv->db_mode) != 0) {
+		g_free (priv->db_mode);
+		priv->db_mode = g_strdup (profile_name);
+	}
+
+	if (tracker_config_get_low_memory_mode (priv->config)) {
+		flags |= TRACKER_DB_MANAGER_LOW_MEMORY_MODE;
+	}
+
+	/* Pause the indexer, this also flushes data to DBs */
+	tracker_indexer_set_running (indexer, FALSE);
+
+	/* Reinitialize DB Manager with new profile */
+	tracker_db_manager_shutdown ();
+
+	if (!tracker_db_manager_init (flags, NULL, FALSE, priv->db_mode)) {
+		g_critical ("Could not restart DB manager, trying again with defaults");
+
+		if (!tracker_db_manager_init (flags, NULL, FALSE, NULL)) {
+			g_critical ("Not even defaults worked, bailing out.");
+			g_assert_not_reached ();
+		}
+
+		/* Reset this if we could get things working with no
+		 * profile set.
+		 */
+		g_free (priv->db_mode);
+		priv->db_mode = NULL;
+
+		return_val = FALSE;
+	}
+
+	/* Restore database pointers */
+	set_up_databases (indexer);
+
+	/* Restart the indexer */
+	tracker_indexer_set_running (indexer, TRUE);
+
+	return return_val;
+}
+
+void
+tracker_indexer_set_profile (TrackerIndexer         *indexer,
+			     const gchar            *profile_name,
+			     DBusGMethodInvocation  *context,
+			     GError                **error)
+{
+	guint request_id;
+
+	tracker_dbus_async_return_if_fail (TRACKER_IS_INDEXER (indexer), context);
+
+	request_id = tracker_dbus_get_next_request_id ();
+	tracker_dbus_request_new (request_id,
+				  "DBus request to switch to profile '%s'", profile_name);
+
+	if (!set_profile (indexer, profile_name)) {
+		GError *actual_error = NULL;
+
+		tracker_dbus_request_failed (request_id,
+					     &actual_error,
+					     "Could not set profile '%s'",
+					     profile_name);
+
+		dbus_g_method_return_error (context, actual_error);
+		g_error_free (actual_error);
+	} else {
+		dbus_g_method_return (context);
+		tracker_dbus_request_success (request_id);
+	}
+}
